"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deprecatedAttr = void 0;
const fast_element_1 = require("@microsoft/fast-element");
const warningQueue = [];
const logTimeout = 5000;
setTimeout(() => {
    if (warningQueue.length) {
        console.warn(`${warningQueue.length} attribute deprecation notice(s):\n ${warningQueue.join('\n')}`);
    }
}, logTimeout);
/**
 * Value converter is an interface from FAST which lets you convert types. An example
 * of this usage is the number converter when using `mode: number` which converts to and from
 * the string attribute on the DOM, to the number property on the class.
 *
 * Instead of doing a conversion we sit in the middle and if a value is set from the attribute
 * we know that the deprecatedName is used, hence we queue a deprecation warning. We only emit one warning
 * per attribute per class, to stop repeat emits on change.
 *
 * this function needs to take all the variables in separately to capture them in the closure rather
 * than pulling them from the `deprecatedConfig` object, as that is modified which causes issues such
 * as infinite loops
 */
const deprecationLogger = (className, deprecatedName, attributeName, converter) => ({
    hasEmitLog: false,
    toView(value) {
        return converter ? converter.toView(value) : value;
    },
    fromView(value) {
        if (value && !this.hasEmitLog) {
            this.hasEmitLog = true;
            warningQueue.push(`Deprecation warning: attribute name "${deprecatedName}" on component "${className}" is deprecated. Please use "${attributeName}" instead.`);
        }
        return converter ? converter.fromView(value) : value;
    },
});
function deprecatedAttr(configOrTarget, prop) {
    let config;
    function decorator($target, $prop) {
        if (arguments.length > 1) {
            // Non invocation:
            // - @attr
            // Invocation with or w/o opts:
            // - @attr()
            // - @attr({...opts})
            config.property = $prop;
        }
        fast_element_1.AttributeConfiguration.locate($target.constructor).push(config);
        if ('deprecatedName' in config) {
            const deprecatedConfig = structuredClone(config);
            deprecatedConfig.converter = deprecationLogger($target.constructor.name, deprecatedConfig.deprecatedName, deprecatedConfig.attribute, deprecatedConfig.converter);
            deprecatedConfig.attribute = config.deprecatedName.toLowerCase();
            fast_element_1.AttributeConfiguration.locate($target.constructor).push(deprecatedConfig);
        }
    }
    if (arguments.length > 1) {
        // Non invocation:
        // - @attr
        config = {};
        decorator(configOrTarget, prop);
        return;
    }
    // Invocation with or w/o opts:
    // - @attr()
    // - @attr({...opts})
    config = configOrTarget === void 0 ? {} : configOrTarget;
    return decorator;
}
exports.deprecatedAttr = deprecatedAttr;
